<?php
namespace Bitfoundry\Etnpay\Controller\Scan;

class Webhook extends \Bitfoundry\Etnpay\Controller\AbstractEtnpay
{
    public function execute()
    {
        $etnpay = $this->_paymentMethod;
        $helper = $this->helper;
        $expectedIp = $etnpay->getConfigData('expected_api_ip');

        if ($expectedIp == "" || $helper->getIp() == $expectedIp || $helper->getIp() == '127.0.0.1') {
            $data = json_decode(json_decode(file_get_contents('php://input')), true);
            //$data = json_decode(file_get_contents('php://input'), true); // simulated payments

            if ($data['payment_id'] == '')
                $orderIdFromEtn = sub_str($data['payment_id'],1); // Magento order reference sent to ETN with an preceding '0'

            $order = $this->getOrderByEtnReference($orderIdFromEtn);
            $etnId = $data['ref'];
            $matchingEtnId = false;

            //payment success
            if ($data['PaymentStatusId'] == 2) {
                if ($order->getBaseTotalDue() != 0) { // prevent double invoicing
                    //check existing invoices for a matching etnID
                    $invoiceCollection = $order->getInvoiceCollection();
                    foreach($invoiceCollection as $invoice) {
                        if ($invoice->getEtnId() == $etnId) {
                            $matchingEtnId = true;
                            break;
                        }
                    }

                    if (!$matchingEtnId) {
                        $etnStatus = $helper->getEtnStatus($orderId);
                        if ($etnConfirmationCode == 2) {
                            $invoice = $order->prepareInvoice();
                            $invoice->setEtnId($etnId);
                            $invoice->register()->capture();

                            $transaction = $this->_transactionFactory->create();
                            $transaction->addObject($invoice)
                                ->addObject($invoice->getOrder())
                                ->save();

                            $status = \Magento\Sales\Model\Order::STATE_PROCESSING;

                            $order->setStatus( $status ); //configure the status
                            $order->setState( $status )->save(); //try and configure the status
                            $order->addStatusHistoryComment(__( 'Invoice created via the webhook, #%1.', $invoice->getIncrementId()));
                            $order->setIsCustomerNotified(true);
                            //$order->sendNewOrderEmail();
                            $order->save();

                            $this->messageManager->addSuccessMessage('Electroneum Payment Successful.');

                            //$invoice->sendEmail(); // send after order save otherwise Invoice Number and date don't come through on email
                        } // end if
                        elseif ($etnConfirmationCode == 1 || $etnConfirmationCode == 7 ) {
                            $status = \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT;
                            $order->setStatus($status);
                            $order->setState($status)->save();
                            $order->addStatusHistoryComment(__('Electroneum payment pending/processing.'));
                            $order->save();
                        } // end if
                        elseif ($etnConfirmationCode == 4) {
                            // payment cancelled
                            $order->addStatusToHistory(Mage_Sales_Model_Order::STATE_CANCELED,'The user cancelled payment in Electroneum app.',false);
                            $order->save();
                        }
                        else {
                            $this->_logger->error("************************************************************************************");
                            $this->_logger->error("*    Payment Tampering");
                            $this->_logger->error("*    Looks like we have some payment redirect tampering on order: " . $order->getIncrementId() . ", key: " . $this->getRequest()->getParam('key') . ", Electroneum.");
                            $this->_logger->error("************************************************************************************");
                        }
                    } // end if
                } // end if
            } // end if
            else {
                // payment failed
                $status = \Magento\Sales\Model\Order::STATE_CANCELED;
                $message = __('Electroneum payment failed. Order cancelled, basket restored.');
                if ($order->getId() && $order->getState() != \Magento\Sales\Model\Order::STATE_CANCELED) {
                    $order->setStatus($status);
                    $order->setState($status)->save();
                    $order->addStatusHistoryComment(__('Payment declined via the Electroneum app.'));
                    $order->registerCancellation($message)->save();
                }
            }
        }
        else {
            $this->_logger->error("************************************************************************************");
            $this->_logger->error("*    Unauthorized IP address trying to access Electroneum payment webhook: " . $helper->getIp());
            $this->_logger->error("************************************************************************************");
        }

        return '';
    }
}
